/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <libspe2.h>
#include <mars/mars.h>

#define INFO	\
"\
MARS Task Barrier Sample					\n\
------------------------					\n\
This program shows the basic usage of the MARS task barrier	\n\
synchronization between host program and multiple MPU programs. \n\
								\n\
This program creates %d tasks and creates a task barrier.	\n\
%d tasks notify the barrier when they finish processing.	\n\
Once those %d tasks notify the barrier and the barrier is	\n\
released, then the remaining %d tasks are allowed to process.	\n\
\n", NUM_TASKS * 2, NUM_TASKS, NUM_TASKS, NUM_TASKS

#define NUM_TASKS	10

extern struct spe_program_handle mpu_task1_prog;
extern struct spe_program_handle mpu_task2_prog;

static struct mars_context *mars_ctx;
static struct mars_task_id task1_id[NUM_TASKS];
static struct mars_task_id task2_id[NUM_TASKS];
static struct mars_task_args task_args;
static struct mars_task_barrier *barrier;

int main(void)
{
	int ret, i;

	printf(INFO);

	ret = mars_context_create(&mars_ctx, 0, 0);
	if (ret) {
		printf("MARS context create failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_task_barrier_create(mars_ctx, &barrier, NUM_TASKS);
	if (ret) {
		printf("MARS task barrier create failed! (%d)\n", ret);
		return 1;
	}

	for (i = 0; i < NUM_TASKS; i++) {
		char name[MARS_TASK_NAME_LEN_MAX];

		snprintf(name, MARS_TASK_NAME_LEN_MAX, "Notify Task %d", i + 1);
		ret = mars_task_create(mars_ctx, &task1_id[i], name,
			mpu_task1_prog.elf_image, MARS_TASK_CONTEXT_SAVE_ALL);
		if (ret) {
			printf("MARS task 1 create failed! (%d)\n", ret);
			return 1;
		}

		task_args.type.u64[0] = mars_ptr_to_ea(barrier);
		ret = mars_task_schedule(&task1_id[i], &task_args, 0);
		if (ret) {
			printf("MARS task 1 schedule failed! (%d)\n", ret);
			return 1;
		}
	}

	for (i = 0; i < NUM_TASKS; i++) {
		char name[MARS_TASK_NAME_LEN_MAX];

		snprintf(name, MARS_TASK_NAME_LEN_MAX, "Wait Task %d", i + 1);
		ret = mars_task_create(mars_ctx, &task2_id[i], name,
			mpu_task2_prog.elf_image, MARS_TASK_CONTEXT_SAVE_ALL);
		if (ret) {
			printf("MARS task 2 create failed! (%d)\n", ret);
			return 1;
		}

		task_args.type.u64[0] = mars_ptr_to_ea(barrier);
		ret = mars_task_schedule(&task2_id[i], &task_args, 0);
		if (ret) {
			printf("MARS task 2 schedule failed! (%d)\n", ret);
			return 1;
		}
	}

	for (i = 0; i < NUM_TASKS; i++) {
		ret = mars_task_wait(&task1_id[i], NULL);
		if (ret) {
			printf("MARS task 1 wait failed! (%d)\n", ret);
			return 1;
		}

		ret = mars_task_wait(&task2_id[i], NULL);
		if (ret) {
			printf("MARS task 2 wait failed! (%d)\n", ret);
			return 1;
		}

		ret = mars_task_destroy(&task1_id[i]);
		if (ret) {
			printf("MARS task 1 destroy failed! (%d)\n", ret);
			return 1;
		}

		ret = mars_task_destroy(&task2_id[i]);
		if (ret) {
			printf("MARS task 2 destroy failed! (%d)\n", ret);
			return 1;
		}
	}

	ret = mars_task_barrier_destroy(barrier);
	if (ret) {
		printf("MARS task barrier destroy failed! (%d)\n", ret);
		return 1;
	}

	ret = mars_context_destroy(mars_ctx);
	if (ret) {
		printf("MARS context destroy failed! (%d)\n", ret);
		return 1;
	}

	return 0;
}
